<?php

namespace Drupal\eav_field\Form;

use Drupal\Core\Form\FormStateInterface;
use Drupal\eav_field\Entity\EavAttribute;
use Drupal\field\FieldStorageConfigInterface;
use Drupal\field_ui\Form\FieldStorageConfigEditForm;

class EavAttributeValueStorageSettingsForm extends FieldStorageConfigEditForm {

  /**
   * {@inheritDoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);

    $attribute = $form_state->get('attribute') /** @var EavAttribute $attribute */;

    $form['#title'] = $this->t('Storage settings of attribute "@attribute"', ['@attribute' => $attribute->getAdministrativeLabel()]);
    $form['#prefix'] = '';

    return $form;
  }

  /**
   * {@inheritDoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    $attribute = $form_state->get('attribute') /** @var EavAttribute $attribute */;

    $field_storage_config = $this->entity; /** @var FieldStorageConfigInterface $field_storage_config */
    $field_storage_config_array = array_intersect_key($field_storage_config->toArray(), array_flip([
      'settings',
      'cardinality',
    ]));

    $attribute->set('value_storage_config', $field_storage_config_array);
    $attribute->save();

    $this->messenger()->addMessage($this->t('Storage settings saved.'));

    if ($attribute->get('value_config')->isEmpty()) {
      $form_state->setRedirect('entity.eav_attribute.value_field_form', ['eav_attribute' => $attribute->id()]);
    }
  }

}
