<?php

namespace Drupal\eav_field\Form;

use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\WidgetInterface;
use Drupal\Core\Field\WidgetPluginManager;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

class EavEntityAttributesForm extends FormBase {

  protected $entityTypeManager;

  protected $entityFieldManager;

  protected $fieldWidgetManager;

  /**
   * {@inheritDoc}
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, EntityFieldManagerInterface $entity_field_manager, WidgetPluginManager $field_widget_manager) {
    $this->entityTypeManager = $entity_type_manager;
    $this->entityFieldManager = $entity_field_manager;
    $this->fieldWidgetManager = $field_widget_manager;
  }

  /**
   * {@inheritDoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('entity_field.manager'),
      $container->get('plugin.manager.field.widget')
    );
  }

  /**
   * {@inheritDoc}
   */
  public function getFormId() {
    return 'eav_entity_attributes_form';
  }

  /**
   * {@inheritDoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $query_parameters = $this->getRouteMatch()->getRawParameters()->all();
    $host_entity_type_id = key($query_parameters);
    $host_entity_id = current($query_parameters);
    $host_entity = $this->entityTypeManager->getStorage($host_entity_type_id)->load($host_entity_id);
    $eav_field_definition = $this->getEntityEavFieldDefinition($host_entity);
    $eav_field_name = $eav_field_definition->getName();

    $eav_widget = $this->fieldWidgetManager->getInstance([
      'field_definition' => $eav_field_definition,
      'configuration' => [
        'type' => 'eav_widget',
      ],
    ]);

    $form['#parents'] = $form['#parents'] ?? [];
    $form[$eav_field_name] = $eav_widget->form($host_entity->get($eav_field_name), $form, $form_state);

    $form['actions'] = [
      '#type' => 'actions',
    ];

    $form['actions']['save'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save'),
      '#button_type' => 'primary',
    ];

    $form['#title'] = $this->t('Editing attributes of "@entity_label"', ['@entity_label' => $host_entity->label()]);

    $form_state->set('host_entity', $host_entity);
    $form_state->set('eav_widget', $eav_widget);

    return $form;
  }

  /**
   * {@inheritDoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $host_entity = $form_state->get('host_entity'); /** @var EntityInterface $host_entity */
    $eav_widget = $form_state->get('eav_widget'); /** @var WidgetInterface $eav_widget */
    $eav_field_definition = $this->getEntityEavFieldDefinition($host_entity);
    $eav_field_name = $eav_field_definition->getName();

    $eav_widget->extractFormValues($host_entity->get($eav_field_name), $form, $form_state);
    $host_entity->save();

    $this->messenger()->addMessage($this->t('Data saved.'));
  }

  /**
   * Return eav field definition.
   *
   * @return FieldDefinitionInterface
   */
  protected function getEntityEavFieldDefinition(EntityInterface $entity) {
    $entity_bundle_field_definitions = $this->entityFieldManager->getFieldDefinitions($entity->getEntityTypeId(), $entity->bundle());
    $eav_field_definition = NULL;

    foreach ($entity_bundle_field_definitions as $entity_bundle_field_definition) {
      if ($entity_bundle_field_definition->getType() == 'eav') {
        $eav_field_definition = $entity_bundle_field_definition;
        break;
      }
    }

    return $eav_field_definition;
  }

}
